import chainer
from chainer import cuda
from chainer import reporter
import numpy as np


CHAINERUI_IMAGE_PREFIX = 'chainerui_image'


class ImageSummary(object):

    def __init__(self):
        self.observation = {}


chainerui_image_observer = ImageSummary()


def image(images, name=None, ch_axis=1, row=0, mode=None, batched=True):
    """summary images to visualize.

    A batch of image is registered on global observation and these images
    are collected by :class:`chainerui.extensions.ImageReporter`. This function
    expects to be used with :class:`chainer.training.Trainer`. If using this
    function without :class:`chainer.training.Trainer`, need to setup
    :class:`chainer.Reporter` before using it.

    Example of how to set arguments::

       >>> from chainerui import summary
       >>>
       >>> x.shape  # = [Batchsize, Channel, Hight, Width]
       (10, 3, 5, 5)
       >>> summary.image(x, name='test')  # images are tiled as 1x10
       >>> summary.image(x, name='test', row=5)  # images are tiled as 2x5
       >>>
       >>> x.shape  # = [C, H, W]
       (3, 5, 5)
       >>> # need to set as a non-batched image and channel axis explicitly
       >>> summary.image(x, name='test', ch_axis=0, batched=False)
       >>>
       >>> x.shape  # = [B, H, W, C]
       (10, 5, 5, 3)
       >>> # need to set channel axis explicitly
       >>> summary.image(x, name='test', ch_axis=-1, row=5)
       >>>
       >>> x.shape  # = [H, W, C]
       (5, 5, 3)
       >>> # need to set as a non-batched image
       >>> summary.image(x, name='test', ch_axis=-1, batched=False)
       >>>
       >>> x.shape  # = [B, H, W], grayscale images
       (10, 5, 5)
       >>> summary.image(x, name='test')  # image are tiled as 1x10
       >>> summary.image(x, name='test', row=5)  # image are tiled as 2x5
       >>>
       >>> x.shape  # = [H, W], a grayscale image
       (5, 5)
       >>> # need to set as a non-bathed image
       >>> summary.image(x, name='test', batched=False)

    Args:
        images (:class:`numpy.ndarray` or :class:`cupy.ndarray` or
            `chainer.Variable`): batch of images. If Number of dimension is
            3 (or 2 when set `batched=False`), the pixels assume as
            black and white image.
        name (str): name of image. when not setting, assigned number
            automatically.
        ch_axis (int): index number of channel dimension. set 1 by default.
            if the images don't have channel axis, this parameter is ignored.
        row (int): row size to visualize batched images. when set 0,
            show on unstuck. if images set only one image, the row size
            will be ignored.
        mode (str): if the images are not RGB or RGBA space, set their
            color space code. ChainerUI supports 'HSV'.
        batched (bool): if the image is not batched, set `False`.
    """

    if isinstance(images, chainer.Variable):
        images = images.data
    images = cuda.to_cpu(images)
    ndim = images.ndim
    if batched:
        if not (ndim == 3 or ndim == 4):
            raise ValueError(
                'Number of array dimension %d must be 3 or 4', ndim)

        if ndim == 4:
            images = _move_ch_to_last(images, ch_axis)
            B, H, W, C = images.shape
            if row == 0:
                row = B
            col = B // row
            images = images.reshape(row, col, H, W, C)
            images = images.transpose(0, 2, 1, 3, 4)
            stuck_image = images.reshape(row * H, col * W, C)
        else:  # ndim == 3
            B, H, W = images.shape
            if row == 0:
                row = B
            col = B // row
            images = images.reshape(row, col, H, W)
            images = images.transpose(0, 2, 1, 3)
            stuck_image = images.reshape(row * H, col * W)
    else:
        if not (ndim == 2 or ndim == 3):
            raise ValueError(
                'Number of array dimension %d must be 2 or 3', ndim)
        if ndim == 3:
            images = _move_ch_to_last(images, ch_axis)
        stuck_image = images

    if name is None:
        # TODO(disktnk): support tupled image and increment automatically
        name = '0'

    current_reporter = reporter.get_current_reporter()
    observer = chainerui_image_observer
    with reporter.report_scope(observer.observation):
        value = {'image': stuck_image}
        if mode is not None:
            value['mode'] = mode.lower()
        current_reporter.report({name: value}, observer)


def _move_ch_to_last(x, axis):
    if axis == -1:
        return x
    rolled_ax = np.append(np.delete(np.arange(x.ndim), axis), axis)
    return x.transpose(rolled_ax)



from collections import OrderedDict
import datetime
import json
import os

from chainerui import DB_SESSION
from chainerui.models.asset import Asset
from chainerui.models.bindata import Bindata


def collect_images(result, assets, force=False):
    """collect images from meta file

    Example of returning structure will be:
      [
        {
          "contents": {
            "name1": "data:image/png;base64,...",
            "name2": "data:image/png;base64,..."
          },
          "train_info" {
            "iteration": 1000,
            "epoch": 1,
            "other_key": "value"
          }
        }, {
          "... (same as above)"
        }
      ]
    """
    path_name = result.path_name
    info_path = os.path.join(path_name, '.chainerui_images')
    start_idx = len(assets)
    if not os.path.isfile(info_path):
        return assets
    file_modified_at = datetime.datetime.fromtimestamp(os.path.getmtime(
        info_path))
    if start_idx > 0:
        if assets[-1].file_modified_at == file_modified_at:
            return assets

    with open(info_path, 'r') as f:
        info_list = json.load(f, object_pairs_hook=OrderedDict)

    if len(info_list) < start_idx:
        start_idx = 0
        assets = []

    for base_info in info_list[start_idx:]:
        image_path = base_info.pop('images')
        asset = Asset.create(
            result_id=result.id, summary=base_info,
            file_modified_at=file_modified_at)
        for key, path in image_path.items():
            with open(os.path.join(path_name, path), 'rb') as f:
                data = f.read()
            content = Bindata(
                asset_id=asset.id, name=path, tag=key, content=data)
            asset.content_list.append(content)
        assets.append(asset)

    DB_SESSION.commit()

    return assets



import argparse
import gevent
from gevent.pywsgi import WSGIServer
import os
import signal

from chainerui import _version
from chainerui import create_app
from chainerui import create_db
from chainerui import DB_FILE_PATH
from chainerui import DB_SESSION
from chainerui.models.project import Project
from chainerui import upgrade_db
from chainerui.utils import db_revision


def _check_db_revision():
    if not db_revision.check_current_db_revision():
        command = 'upgrade'
        if db_revision.current_db_revision() is None:
            command = 'setup'
        msg = 'The current DB schema version is not supported, ' +\
            'please %s DB' % command
        print(msg)
        return False
    return True


def server_handler(args):
    """server_handler."""
    if not _check_db_revision():
        return

    app = create_app()
    if args.debug:
        # start server with:
        # - env: development
        # - debug: on
        app.config['ENV'] = 'development'
        app.run(host=args.host, port=args.port, debug=True, threaded=True)
    else:
        # start server with:
        # - env: production
        # - debug: off
        listener = '{:s}:{:d}'.format(args.host, args.port)
        http_server = WSGIServer(listener, application=app)

        def stop_server():
            if http_server.started:
                http_server.stop()

        gevent.signal(signal.SIGTERM, stop_server)
        gevent.signal(signal.SIGINT, stop_server)

        print(' * Environment: production')
        print(' * Running on http://{}/ (Press CTRL+C to quit)'
              .format(listener))

        try:
            http_server.serve_forever()
        except (KeyboardInterrupt, SystemExit):
            stop_server()


def db_handler(args):
    """db_handler."""

    if args.type == 'create':
        create_db()

    if args.type == 'status':
        current_rev = db_revision.current_db_revision()
        print('current_rev', current_rev)

    if args.type == 'upgrade':
        upgrade_db()

    if args.type == 'revision':
        db_revision.new_revision()

    if args.type == 'drop':
        if os.path.exists(DB_FILE_PATH):
            os.remove(DB_FILE_PATH)


def project_create_handler(args):
    """project_create_handler."""
    if not _check_db_revision():
        return

    project_path = os.path.abspath(args.project_dir)
    project_name = args.project_name

    project = DB_SESSION.query(Project).\
        filter_by(path_name=project_path).first()

    if project is None:
        project = Project.create(project_path, project_name)
    else:
        print('Pathname already registered.')


def create_parser():
    parser = argparse.ArgumentParser(description='chainerui command')
    parser.add_argument(
        '--version', '-v', action='version', version=_version.__version__)
    subparsers = parser.add_subparsers()

    # chainerui server
    parser_server = subparsers.add_parser(
        'server', help='see `chainerui server -h`')
    parser_server.add_argument(
        '-H', '--host', required=False, help='host', default='localhost')
    parser_server.add_argument(
        '-p', '--port', required=False, type=int, help='port', default=5000)
    parser_server.add_argument(
        '-d', '--debug', action='store_true', help='debug')
    parser_server.set_defaults(
        handler=server_handler)

    # chainerui db
    parser_db = subparsers.add_parser(
        'db', help='see `chainerui db -h`')
    parser_db.add_argument(
        'type', choices=['create', 'drop', 'status', 'upgrade', 'revision'])
    parser_db.set_defaults(
        handler=db_handler)

    # chainerui project
    parser_project = subparsers.add_parser(
        'project', help='see `chainerui project -h`')
    parser_project_sub = parser_project.add_subparsers()

    # chainerui project create
    parser_project_create = parser_project_sub.add_parser(
        'create', help='see `chainerui project create -h`')
    parser_project_create.add_argument(
        '-d', '--project-dir', required=True, type=str, help='project-dir')
    parser_project_create.add_argument(
        '-n', '--project-name', type=str, help='project-name', default=None)
    parser_project_create.set_defaults(
        handler=project_create_handler)

    return parser


def main():
    """main."""
    parser = create_parser()
    args = parser.parse_args()

    if hasattr(args, 'handler'):
        args.handler(args)
    else:
        parser.print_help()


if __name__ == '__main__':
    main()



from collections import OrderedDict
import hashlib
import json
import os
import shutil
import warnings

from chainer.training import extension
from chainer.training import trigger as trigger_module
import numpy as np

from chainerui import summary
from chainerui.utils import tempdir


try:
    from PIL import Image  # NOQA

    _available = True

except (ImportError, TypeError):
    _available = False


def _check_available():
    if not _available:
        warnings.warn('Pillow is not installed on your environment, '
                      'so no image will be output at this time.'
                      'Please install Pillow to save images.\n\n'
                      '  $ pip install Pillow\n')


class ImageReport(extension.Extension):
    """Reports images for visualizing

    This extension supports visualizing image data on ChainerUI.
    """

    def __init__(self, trigger=(1, 'epoch'), image_generator=None):
        _check_available()
        self._trigger = trigger_module.get_trigger(trigger)
        self._fn = image_generator
        self._info_name = '.chainerui_images'
        self._infos = []

    @staticmethod
    def available():
        _check_available()
        return _available

    def initialize(self, trainer):
        observer = summary.chainerui_image_observer
        trainer.reporter.add_observer(
            summary.CHAINERUI_IMAGE_PREFIX, observer)
        self._observer = observer

    def __call__(self, trainer):
        if not _available:
            return
        if not self._trigger(trainer):
            return
        if self._fn is not None:
            self._fn(trainer)

        image_prefix = summary.CHAINERUI_IMAGE_PREFIX
        observations = self._observer.observation
        pooled_images = OrderedDict()
        keys = observations.keys()
        for k in list(keys):
            if k.startswith(image_prefix):
                pooled_images[k[len(image_prefix) + 1:]] = observations.pop(k)
        if len(pooled_images) == 0:
            return

        out_path = trainer.out
        updater = trainer.updater
        image_path = {}
        for key, value in pooled_images.items():
            image = self._normalize_8bit(value['image'])
            file_name = 'iter_%d_%s.png' % (
                updater.iteration, self._get_hash(key))
            file_path = os.path.join(out_path, file_name)
            self._save_image(image, file_path, mode=value.get('mode', None))
            image_path[key] = file_name
        info = {
            'epoch': updater.epoch,
            'iteration': updater.iteration,
            'images': image_path,
        }
        self._infos.append(info)

        with tempdir(prefix=self._info_name, dir=out_path) as tempd:
            path = os.path.join(tempd, self._info_name)
            with open(path, 'w') as f:
                json.dump(self._infos, f, indent=4)

            shutil.move(path, os.path.join(out_path, self._info_name))

    def _get_hash(self, key):
        return hashlib.md5(key.encode('utf-8')).hexdigest()[:8]

    def _normalize_8bit(self, array):
        if array.dtype == np.uint8:
            return array
        return np.asarray(np.clip(array * 255, 0.0, 255.0), dtype=np.uint8)

    def _save_image(self, img, name, ext='PNG', mode=None):
        if mode is None:
            Image.fromarray(img).save(name, format=ext)
        elif mode == 'hsv':
            Image.fromarray(img, mode='HSV').convert('RGB').save(
                name, format=ext)



import datetime
import json
import os

from chainerui import DB_SESSION
from chainerui.models.argument import Argument
from chainerui.models.command import Command
from chainerui.models.log import Log
from chainerui.models.snapshot import Snapshot
from chainerui.utils import is_numberable


def load_result_json(result_path, json_file_name):
    """load_result_json."""
    json_path = os.path.join(result_path, json_file_name)

    _list = []
    if os.path.isfile(json_path):
        with open(json_path) as json_data:
            try:
                _list = json.load(json_data)
            except ValueError as err:
                print("Failed to load json: {}, {}".format(json_path, err))

    return _list


def crawl_result_path(result_path, include_log):
    """crawl_result_path."""
    result = {
        'logs': [],
        'args': [],
        'commands': [],
        'snapshots': []
    }

    if os.path.isdir(result_path):
        if include_log:
            result['logs'] = load_result_json(result_path, 'log')
        result['args'] = load_result_json(result_path, 'args')
        result['commands'] = load_result_json(result_path, 'commands')

        snapshots = [
            x for x in os.listdir(result_path) if x.count('snapshot_iter_')
        ]
        snapshots.sort()
        result['snapshots'] = snapshots

    return result


def _check_log_updated(result):
    log_json_path = os.path.join(result.path_name, 'log')
    if not os.path.isfile(log_json_path):
        # log file is removed, so don't have to update
        return False

    current_modified_at = result.log_modified_at
    modified_at = datetime.datetime.fromtimestamp(
        os.path.getmtime(log_json_path))
    if current_modified_at is None or current_modified_at != modified_at:
        result.log_modified_at = modified_at
        return True

    return False


def crawl_result(result, force=False):
    """crawl_results."""
    now = datetime.datetime.now()

    if (not force) and (now - result.updated_at).total_seconds() < 4:
        return result

    # if log file is not updated, not necessary to get log contents
    is_updated = _check_log_updated(result)
    crawled_result = crawl_result_path(result.path_name, is_updated)

    if is_updated:
        current_log_idx = len(result.logs)
        if len(crawled_result['logs']) < current_log_idx:
            current_log_idx = 0
            result.logs = []
            result.args = None
        for log in crawled_result['logs'][current_log_idx:]:
            result.logs.append(Log(log))

    if result.args is None:
        result.args = Argument(json.dumps(crawled_result['args']))

    result.commands = []
    result.snapshots = []

    for cmd in crawled_result['commands']:
        result.commands.append(Command(**cmd))

    for snapshot in crawled_result['snapshots']:
        number_str = snapshot.split('snapshot_iter_')[1]
        if is_numberable(number_str):
            result.snapshots.append(
                Snapshot(snapshot, int(number_str))
            )

    result.updated_at = datetime.datetime.now()
    DB_SESSION.commit()

    return result



from flask import jsonify
from flask.views import MethodView

from chainerui import DB_SESSION
from chainerui.models.asset import Asset
from chainerui.models.project import Project
from chainerui.models.result import Result
from chainerui.tasks.collect_images import collect_images


class ResultImageAPI(MethodView):

    def get(self, result_id=None, project_id=None):
        # project is not necessary to collect images,
        # but check parent project exists or not
        project = DB_SESSION.query(Project).\
            filter_by(id=project_id).\
            first()
        if project is None:
            return jsonify({
                'project': None,
                'message': 'No interface defined for URL.'
            }), 404

        result = DB_SESSION.query(Result).\
            filter_by(id=result_id).\
            filter_by(is_unregistered=False).\
            first()
        if result is None:
            return jsonify({
                'result': None,
                'message': 'No interface defined for URL.'
            }), 404

        assets = DB_SESSION.query(Asset).\
            filter_by(result_id=result_id).\
            order_by(Asset.id).all()
        if assets is None:
            assets = []

        images = collect_images(result, assets)

        return jsonify({'images': [image.serialize for image in images]})



from flask import jsonify
from flask import request
from flask.views import MethodView

from chainerui import DB_SESSION
from chainerui.models.project import Project
from chainerui.models.result import Result
from chainerui.tasks import collect_results
from chainerui.tasks import crawl_result


class ResultAPI(MethodView):
    """ResultAPI."""

    def get(self, id=None, project_id=None):
        """get."""
        logs_limit = request.args.get('logs_limit', default=-1, type=int)

        project = DB_SESSION.query(Project).\
            filter_by(id=project_id).\
            first()
        if project is None:
            return jsonify({
                'project': None,
                'message': 'No interface defined for URL.'
            }), 404

        if id is None:

            collect_results(project)

            results = DB_SESSION.query(Result).\
                filter_by(project_id=project_id).\
                filter_by(is_unregistered=False).\
                all()

            for result in results:
                result = crawl_result(result)

            return jsonify({
                'results': [
                    r.serialize_with_sampled_logs(logs_limit) for r in results
                ]
            })

        else:

            result = DB_SESSION.query(Result).\
                filter_by(id=id).\
                filter_by(is_unregistered=False).\
                first()

            if result is None:
                return jsonify({
                    'result': None,
                    'message': 'No interface defined for URL.'
                }), 404

            result = crawl_result(result)

            return jsonify({
                'result': result.serialize_with_sampled_logs(logs_limit)
            })

    def put(self, id, project_id=None):
        """put."""
        result = DB_SESSION.query(Result).filter_by(id=id).first()
        if result is None:
            response = jsonify({
                'result': None, 'message': 'No interface defined for URL.'
            })
            return response, 404

        request_json = request.get_json()
        request_result = request_json.get('result')

        name = request_result.get('name', None)
        if name is not None:
            result.name = name

        is_unregistered = request_result.get('isUnregistered', None)
        if is_unregistered is not None:
            result.is_unregistered = is_unregistered

        DB_SESSION.add(result)
        DB_SESSION.commit()

        return jsonify({'result': result.serialize})

    def delete(self, id, project_id=None):
        """delete."""
        result = DB_SESSION.query(Result).filter_by(id=id).first()
        if result is None:
            response = jsonify({
                'result': None, 'message': 'No interface defined for URL.'
            })
            return response, 404

        DB_SESSION.delete(result)
        DB_SESSION.commit()

        return jsonify({'result': result.serialize})



import datetime

from sqlalchemy import Column
from sqlalchemy import DateTime
from sqlalchemy import Integer
from sqlalchemy.orm import relationship
from sqlalchemy import String

from chainerui import DB_BASE
from chainerui import DB_SESSION
from chainerui.tasks.collect_results import collect_results


class Project(DB_BASE):
    """Project Model."""
    __tablename__ = 'project'

    id = Column(Integer, primary_key=True)
    path_name = Column(String(512), unique=True)
    name = Column(String(512))
    results = relationship('Result', cascade='all, delete-orphan')
    created_at = Column(DateTime, default=datetime.datetime.now())
    updated_at = Column(
        DateTime,
        default=datetime.datetime.now()
    )

    def __init__(self, path_name=None, name=None):
        self.path_name = path_name
        self.name = name

    def __repr__(self):
        return '<Project id: %r, path_name: %r />' % (self.id, self.path_name)

    @classmethod
    def create(cls, path_name=None, name=None):
        """initialize an instance and save it to db."""

        project = cls(path_name, name)

        DB_SESSION.add(project)
        DB_SESSION.commit()

        collect_results(project, force=True)
        return project

    @property
    def serialize(self):
        """serialize."""
        return {
            'id': self.id,
            'pathName': self.path_name,
            'name': self.name
        }



import errno
import os

from alembic.command import upgrade
from alembic.config import Config
from flask import Flask
from flask import jsonify
from flask import render_template
from flask import send_from_directory
from flask import url_for
from sqlalchemy import create_engine
from sqlalchemy.exc import OperationalError
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import scoped_session
from sqlalchemy.orm import sessionmaker

from chainerui import _version


__version__ = _version.__version__


CHAINERUI_ENV = os.getenv('CHAINERUI_ENV', 'production')
CHAINERUI_ROOT = os.path.abspath(
    os.path.expanduser(os.getenv('CHAINERUI_ROOT', '~/.chainerui')))
PACKAGE_DIR = os.path.abspath(os.path.dirname(__file__))
DB_FILE_DIR = os.path.join(CHAINERUI_ROOT, 'db')
DB_FILE_NAME = 'chainerui_test.db' if CHAINERUI_ENV == 'test' \
    else 'chainerui.db'
DB_FILE_PATH = os.path.join(DB_FILE_DIR, DB_FILE_NAME)
SQLALCHEMY_DATABASE_URI = 'sqlite:///' + DB_FILE_PATH
ENGINE = create_engine(
    SQLALCHEMY_DATABASE_URI,
    convert_unicode=True,
    connect_args={'check_same_thread': False},
    echo=(CHAINERUI_ENV == 'development')
)
DB_BASE = declarative_base()
DB_SESSION = scoped_session(
    sessionmaker(autocommit=False, autoflush=False, bind=ENGINE)
)


def get_db_migration_config():
    ini_path = os.path.join(PACKAGE_DIR, 'alembic.ini')
    config = Config(ini_path)
    config.set_main_option(
        "script_location", os.path.join(PACKAGE_DIR, 'migration'))
    return config


def create_db():
    """create_db."""
    try:
        os.makedirs(DB_FILE_DIR)
    except OSError as exception:
        if exception.errno == errno.EEXIST and os.path.isdir(DB_FILE_DIR):
            pass
        else:
            raise
    print('DB_FILE_PATH: ', DB_FILE_PATH)


def upgrade_db():
    """upgrade_db."""
    if not os.path.isdir(DB_FILE_DIR):
        print('DB is not initialized, please run \'create\' command before')
        return
    config = get_db_migration_config()
    upgrade(config, 'head')


def create_db_session():
    """create_db_session."""
    session = scoped_session(
        sessionmaker(autocommit=False, autoflush=False, bind=ENGINE)
    )
    return session()


def create_app():
    """create_app."""

    app = Flask(__name__)
    app.config['JSONIFY_PRETTYPRINT_REGULAR'] = False

    def dated_url_for(endpoint, **values):
        """dated_url_for."""
        if endpoint == 'static':
            filename = values.get('filename', None)
            if filename:
                file_path = os.path.join(app.root_path, endpoint, filename)
                values['_'] = int(os.stat(file_path).st_mtime)
        return url_for(endpoint, **values)

    @app.context_processor
    def override_url_for():
        """override_url_for."""
        return dict(url_for=dated_url_for)

    @app.teardown_appcontext
    def shutdown_session(exception=None):
        DB_SESSION.remove()

    @app.route('/')
    @app.route('/projects/<int:project_id>')
    @app.route('/projects/<int:project_id>/results/<int:result_id>')
    @app.route('/projects/<int:project_id>/results/<int:result_id>/images')
    def index(**kwargs):
        """render react app."""
        return render_template('index.html')

    @app.route('/favicon.ico')
    def favicon():
        return send_from_directory(
            os.path.join(app.root_path, 'static', 'dist'),
            'favicon.ico', mimetype='image/vnd.microsoft.icon')

    # error handling
    @app.errorhandler(OperationalError)
    def handle_invalid_usage(error):
        """handle errors caused by db query."""
        print('caught exception from db:', error.args)
        response = jsonify({
            'error': {
                'type': 'DBOperationalError',
                'message': 'Failed to send request to the database.'
            }
        })
        response.status_code = 400  # Bad Request

        return response

    from chainerui.views.project import ProjectAPI
    from chainerui.views.result import ResultAPI
    from chainerui.views.result_command import ResultCommandAPI
    from chainerui.views.result_image import ResultImageAPI

    project_resource = ProjectAPI.as_view('project_resource')
    result_resource = ResultAPI.as_view('result_resource')
    result_command_resource = ResultCommandAPI.as_view(
        'result_command_resource')
    result_image_resource = ResultImageAPI.as_view('result_image_resource')

    # project API
    app.add_url_rule(
        '/api/v1/projects',
        defaults={'id': None}, view_func=project_resource, methods=['GET'])
    app.add_url_rule(
        '/api/v1/projects', view_func=project_resource, methods=['POST'])
    app.add_url_rule(
        '/api/v1/projects/<int:id>',
        view_func=project_resource, methods=['GET', 'PUT', 'DELETE'])

    # result API
    app.add_url_rule(
        '/api/v1/projects/<int:project_id>/results',
        defaults={'id': None}, view_func=result_resource, methods=['GET'])
    app.add_url_rule(
        '/api/v1/projects/<int:project_id>/results/<int:id>',
        view_func=result_resource, methods=['GET', 'PUT', 'DELETE'])

    # result command API
    app.add_url_rule(
        '/api/v1/projects/<int:project_id>/results/<int:result_id>/commands',
        view_func=result_command_resource, methods=['POST'])

    # result image API
    app.add_url_rule(
        '/api/v1/projects/<int:project_id>/results/<int:result_id>/images',
        view_func=result_image_resource, methods=['GET'])

    return app



# -*- coding: utf-8 -*-
import os
from setuptools.command.sdist import sdist
from setuptools import find_packages
from setuptools import setup
import subprocess

with open('requirements.txt') as f:
    required = f.read().splitlines()


here = os.path.abspath(os.path.dirname(__file__))
# Get __version__ variable
exec(open(os.path.join(here, 'chainerui', '_version.py')).read())


class chainerui_sdist(sdist):
    def run(self):
        subprocess.check_call('cd frontend && npm run build', shell=True)
        sdist.run(self)


setup(
    name='chainerui',
    version=__version__,  # NOQA
    description='ChainerUI: User Interface for Chainer',
    install_requires=required,
    package_data={
        'chainerui': [
            'templates/*',
            'static/**/*'
        ],
    },
    author='',
    author_email='',
    url='https://github.com/chainer/chainerui',
    packages=find_packages(exclude=('tests', 'docs')),
    entry_points={
        "console_scripts": [
            "chainerui=chainerui.app:main",
        ]
    },
    tests_require=['pytest'],
    cmdclass={
        'sdist': chainerui_sdist
    },
)
